package org.devcake.groovy.remoteconsole

import org.devcake.groovy.autoimports.AutoImportsGroovyShell

/**
 * Created by IntelliJ IDEA.
 * User: Admin
 * Date: 29.08.2010
 * Time: 21:48:20
 * To change this template use File | Settings | File Templates.
 */
class RemoteConsoleServer {
    public static final String END_LINE_OUTPUT_SUFFIX = '<<<OUTPUT'
    public static final String END_LINE_ERROR_SUFFIX = '<<<ERROR'

    int port

    ServerSocket serverSocket
    ShellConfigurationFactory shellConfigurationFactory = new DefaultShellConfigurationFactory()
    String encoding = System.getProperty("file.encoding")

    RemoteConsoleServer(int port) {
        this.port = port
    }

    void start() {
        serverSocket = new ServerSocket(port)
        Thread.start {
            while (!serverSocket.isClosed()) {
                try {
                    serverSocket.accept { clientSocket ->
                        processClient(clientSocket)
                    }
                } catch (SocketException e) {
                    // we ignore exception that occurs due to closed connection, and rethrow any other exception
                    if (!serverSocket.isClosed()) throw e
                }
            }

        }
    }

    /**
     * This method was made protected as a workaround for some strange Groovy dispatching bug.
     * Otherwise subclasses were unable to resolve this method call. 
     */
    protected void processClient(Socket clientSocket) {
        clientSocket.withStreams { inputStream, outputStream ->
            try {
                def outWriter = new PrintWriter(outputStream, true)
                def inReader = new BufferedReader(new InputStreamReader(inputStream, encoding))
                def xmlLines = []
                def inputLine
                while ((inputLine = inReader.readLine()) != '</execute>') {
                    xmlLines << inputLine
                }
                xmlLines << inputLine
                def xml = new XmlParser().parseText(xmlLines.join('\n'))
                def options = [:]
                xml.options.option.each{options[it.@name] = it.@value}
                def oldOut = System.out
                def oldErr = System.err
                try {
                    def shell = createShell(options)
                    System.out = new PrintStream(new EndLineFilterOutputStream(outputStream, END_LINE_OUTPUT_SUFFIX), true)
                    System.err = new PrintStream(new EndLineFilterOutputStream(outputStream, END_LINE_ERROR_SUFFIX), true)
                    def result = shell.runSelected(xml.fullScript.text(), xml.selectedScript.text(), 'someScriptName.groovy', [])
                    outWriter.println(xml.returnSeparator.text())
                    outWriter.println(result)
                } catch (Throwable e) {
                    outWriter.println(xml.errorSeparator.text())
                    e.printStackTrace(outWriter)
                } finally {
                    System.out = oldOut
                    System.err = oldErr
                }
            } catch (Throwable e) {
                e.printStackTrace()
                throw e
            }
        }
    }

    /**
     * We return object type instead of Shell intentionally, so that this code
     * can return shell created by different classloaders if necessary
     * @param options
     * @return instance of groovy.lang.GroovyShell
     */
    protected def createShell(Map options) {
        def shellConfiguration = shellConfigurationFactory.createShellConfiguration(options)
        return new AutoImportsGroovyShell(shellConfiguration.classLoader, shellConfiguration.binding)
    }

    void stop() {
        if (serverSocket) {
            serverSocket.close()
        }
    }

    boolean isRunning() {
        serverSocket != null && !serverSocket.isClosed()
    }
}
